"""django-lrucache-backend - A smarter local memory cache backend for Django"""

from .backend import LRUObjectCache

__version__ = "3.0.1"
__author__ = "Josh Smeaton <josh.smeaton@gmail.com>"
__all__ = ["LRUObjectCache"]
